// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institut fuer Eisenforschung GmbH
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxrepo.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_PW_OVERLAP_H_
#define _SX_PW_OVERLAP_H_

#include <SxOverlap.h>
#include <SxDiracLib.h>

/** \brief Plane-wave overlap operator interface

    \author C. Freysoldt, freysoldt@mpie.de */
class SX_EXPORT_DIRAC SxPWOverlap : public SxOverlapBase
{
   public:
      /// Norm square
      virtual double normSqr (const SxVecRef<SxComplex16> &psi) const;

      /// Scalar product
      virtual SxComplex16 dot (const SxVecRef<SxComplex16> &x,
                               const SxVecRef<SxComplex16> &y) const;

      /// Apply overlap operator
      virtual SxVecRef<SxComplex16>
      apply (const SxVecRef<SxComplex16> &psi) const;
      
      /// Set states x orthogonal to states y
      virtual void setOrthogonal (SxVecRef<SxComplex16> *xPtr,
                                  const SxVecRef<SxComplex16> &y) const;

      /// Orthonormalize states x
      virtual void orthonormalize (SxVecRef<SxComplex16> *xPtr,
                                   const SxOrthoMethod how=GramSchmidt) const;
             
};

#endif /* _SX_PW_OVERLAP_H_ */
